--*********************************************************************************
--* This file is part of the RIBit Renderman(tm) exporter for max

--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************

try (destroyDialog ro_RIBit_Setup)catch()

rollout ro_RIBit_Setup "RIBit 0.1 Setup"
(
	global RIBit_Settings = s_RIBit_Settings()


	-- User Interface
	-------------------
	group "Renderers Setup " (
		listBox lbRenderersList "" items:#() selection:0 height:6
		button bnRendAdd "Add" across:4 width:50 height:16 offset:[-10,0]
		button bnRendDel "Remove" width:50 height:16 offset:[-20,0]
		button bnRendMoveUp "Up" width:50 height:16 offset:[20,0]
		button bnRendMoveDown "Down" width:50 height:16 offset:[10,0]
		editText edRendererName "Name:" align:#left enabled:false
		editText edRendererPath "Path:" across:2 fieldWidth:225 align:#left enabled:false
		button bnRendPathBrowse "..." width:16 height:16 align:#right enabled:false tooltip:"Browse..."
		editText edRendererParams "Add. Parameters:" fieldWidth:190
		editText edRendererSLC "SL Compiler:" across:2 fieldWidth:190 align:#left enabled:false
		button bnRendSLCBrowse "..." width:16 height:16 align:#right enabled:false tooltip:"Browse..."
		editText edRendererSLCExt "Compiled SL Extension:" fieldWidth:40 align:#left enabled:false
	)
	
	group "Directories Setup " (
		label lblRIBDir "RIB files:" across:3 align:#left
		editText edRIBDir "" align:#left fieldWidth:190 offset:[-30,0]
		button bnRIBDirBrowse "..." width:16 height:16 align:#right tooltip:"Browse..."
		label lblShadDir "Shader files:" across:3 align:#left
		editText edShadDir "" align:#left fieldWidth:190 offset:[-30,0]
		button bnShadDirBrowse "..." width:16 height:16 align:#right tooltip:"Browse..."
		label lblTextureDir "Texture files:" across:3 align:#left
		editText edTextureDir "" align:#left fieldWidth:190 offset:[-30,0]
		button bnTextureDirBrowse "..." width:16 height:16 align:#right tooltip:"Browse..."
		label lblImageDir "Image files:" across:3 align:#left
		editText edImageDir "" align:#left fieldWidth:190 offset:[-30,0]
		button bnImageDirBrowse "..." width:16 height:16 align:#right tooltip:"Browse..."
	)
	



	-- Functions
	------------------------
	fn getRenderersList =
	(
		lbRenderersList.items = for i in RIBit_Settings.renderers collect (i.name + "  (" + i.path + ")")
	)
	
	
	
fn updateUI =
	(
		getRenderersList()

		edRendererName.enabled = lbRenderersList.selection > 0
		edRendererPath.enabled = lbRenderersList.selection > 0
		bnRendPathBrowse.enabled = lbRenderersList.selection > 0
		edRendererParams.enabled = lbRenderersList.selection > 0

		edRendererSLC.enabled = lbRenderersList.selection > 0
		bnRendSLCBrowse.enabled = lbRenderersList.selection > 0
		edRendererSLCExt.enabled = lbRenderersList.selection > 0

		bnRendDel.enabled = lbRenderersList.selection > 0
		bnRendMoveUp.enabled = lbRenderersList.selection > 0
		bnRendMoveDown.enabled = lbRenderersList.selection > 0
		
		if lbRenderersList.selection > 0 then (
			edRendererName.text = RIBit_Settings.renderers[lbRenderersList.selection].name
			edRendererPath.text = RIBit_Settings.renderers[lbRenderersList.selection].path
			edRendererParams.text = RIBit_Settings.renderers[lbRenderersList.selection].cmdLineParams

			edRendererSLC.text = RIBit_Settings.renderers[lbRenderersList.selection].slCompiler
			edRendererSLCExt.text = RIBit_Settings.renderers[lbRenderersList.selection].compiledSLExt
		)
		
		edRIBDir.text = RIBit_Settings.RIBDir
		edShadDir.text = RIBit_Settings.ShaderDir
		edTextureDir.text = RIBit_Settings.textureDir
		edImageDir.text = RIBit_Settings.ImageDir
	)
	
	fn init =
	(
		RIBit_Settings.readFromFile()
		updateUI()
	)

	-- Event Handlers
	
------------------------
	
on lbRenderersList selected val do (
		updateUI()
	)
	
	on 
edRendererName entered text do (
		RIBit_Settings.renderers[lbRenderersList.selection].name = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	
	on edRendererPath entered text do (
		RIBit_Settings.renderers[lbRenderersList.selection].path = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	
	on bnRendPathBrowse pressed do (
	
	local fname = getOpenFileName caption:("Select Application for \"" + edRendererName.text + "\"") types:"Applications|*.exe;*.com;*.bat|All Files (*.*)|*.*"
		if fname != undefined then (
			edRendererPath.text = fname
			RIBit_Settings.renderers[lbRenderersList.selection].path = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)
	
	on edRendererParams entered text do (
		RIBit_Settings.renderers[lbRenderersList.selection].cmdLineParams = text
		RIBit_Settings.writeToFile()
		updateUI()
	)



	on edRendererSLC entered text do (
		RIBit_Settings.renderers[lbRenderersList.selection].slCompiler = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	
	on bnRendSLCBrowse pressed do (
	
	local fname = getOpenFileName caption:("Select SL-Compiler Application for \"" + edRendererName.text + "\"") types:"Applications|*.exe;*.com;*.bat|All Files (*.*)|*.*"
		if fname != undefined then (
			edRendererSLC.text = fname
			RIBit_Settings.renderers[lbRenderersList.selection].slCompiler = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)


	on edRendererSLCExt entered text do (
		RIBit_Settings.renderers[lbRenderersList.selection].compiledSLExt = text
		RIBit_Settings.writeToFile()
		updateUI()
	)


	
on bnRendAdd pressed do (
		RIBit_Settings.addRenderer "New Renderer" "" "" "" ""
		getRenderersList()
		lbRenderersList.selection = lbRenderersList.items.count
		updateUI()
	)
	
	on bnRendDel pressed do (
		RIBit_Settings.removeRenderer lbRenderersList.selection
		getRenderersList()
		lbRenderersList.selection = 0
		RIBit_Settings.writeToFile()
	)
	
	on bnRendMoveUp pressed do (
		local i = lbRenderersList.selection
		if i > 1 then (
			local item = RIBit_Settings.renderers[i]
			RIBit_Settings.removeRenderer i
			insertItem item RIBit_Settings.renderers (i - 1)
			getRenderersList()
			lbRenderersList.selection = i - 1
			RIBit_Settings.writeToFile()
		)
	)
	
	on bnRendMoveDown pressed do (
		local i = lbRenderersList.selection
		if i < lbRenderersList.items.count then (
			local item = RIBit_Settings.renderers[i]
			RIBit_Settings.removeRenderer i
			insertItem item RIBit_Settings.renderers (i + 1)
			getRenderersList()
			lbRenderersList.selection = i + 1
			RIBit_Settings.writeToFile()
		)
	)





	on edRIBDir entered text do (
		RIBit_Settings.RIBDir = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	on bnRIBDirBrowse pressed do (
		local fname = getSavePath()
		if fname != undefined then (
			edRIBDir.text = fname
			RIBit_Settings.RIBDir = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)
	
	on edShadDir entered text do (
		RIBit_Settings.ShaderDir = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	on bnShadDirBrowse pressed do (
		local fname = getSavePath()
		if fname != undefined then (
			edShadDir.text = fname
			RIBit_Settings.ShaderDir = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)
	
	on edTextureDir entered text do (
		RIBit_Settings.textureDir = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	on bnTextureDirBrowse pressed do (
		local fname = getSavePath()
		if fname != undefined then (
			edTextureDir.text = fname
			RIBit_Settings.textureDir = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)

	on edImageDir entered text do (
		RIBit_Settings.ImageDir = text
		RIBit_Settings.writeToFile()
		updateUI()
	)
	on bnImageDirBrowse pressed do (
		local fname = getSavePath()
		if fname != undefined then (
			edImageDir.text = fname
			RIBit_Settings.ImageDir = fname
			RIBit_Settings.writeToFile()
			updateUI()
		)
	)


	on ro_RIBit_Setup open do init()

) -- end of ro_RIBit_Setup rollout


createDialog ro_RIBit_Setup width:300