--*********************************************************************************
--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************


struct s_RIBit_Camera (
	cam,
	settings = s_RIBit_Settings(),
	indent = "",
	fileHandle,
	
	

	fn exportRIBBox =
	(
		if isValidNode cam then (
			local modIndex = s_RIBit_Functions.findModifier cam mo_RIBit_RIBBox multiple:true
			for i in modIndex do (
				format "%##---- RIB Box ----##\n" indent to:fileHandle
				local lines = filterString (s_RIBit_Functions.parseRIBBox cam.modifiers[i].RIBCode params:#(cam)) "\n\r"
				for l in lines do
					format "%%\n" indent l to:fileHandle
			)
		)
	),



	fn expotLightAsCamera =
	(
		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		local ribProjection = s_RIBit_Option "Projection" indent:indent fileHandle:fileHandle
		local nearClip = 0.1
		local farClip = 100000.0
		if isKindOf cam TargetDirectionallight or isKindOf cam Directionallight then (
			ribProjection.addParameter "orthographic" undefined
			ribMotion.attribute = ribProjection
			ribMotion.write()
			local xmin = -cam.falloff
			local xmax = cam.falloff
			local ymin = -(cam.falloff / cam.aspect)
			local ymax = (cam.falloff / cam.aspect)
			Format "%ScreenWindow % % % %\n" indent xmin xmax ymin ymax to:fileHandle
		) else (
			ribProjection.addParameter "perspective" undefined

			fn motionFunc params: =
			(
				local cameraVFOV = if (isKindOf params[2] freeSpot or isKindOf params[2] targetSpot) then params[2].falloff else 95
				params[1].changeValue "fov" cameraVFOV
				params[1].write()
			)
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribProjection, cam)
			ribMotion.dependsOn = #(cam.fov.isAnimated)
			ribMotion.bypass = settings.useConservativeMotion
			ribMotion.write()
		)
		Format "%Clipping % %\n" indent nearClip farClip to:fileHandle

		fn motionFunc params: =
		(
			params[1].transform = params[2].transform
			params[1].inversePOV()
			params[1].write()
		)
		local ribTransform = s_RIBit_Transform indent:indent fileHandle:fileHandle
		ribMotion.func = motionFunc
		ribMotion.funcParams = #(ribTransform, cam)
		ribMotion.dependsOn = #(cam.transform.isAnimated)
		ribMotion.bypass = false
		ribMotion.write()
	),



	fn exportCamera =
	(
		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		local renderAspect = (settings.rendWidth as float)/settings.rendHeight
		local ribTransform = s_RIBit_Transform (cam.transform) indent:indent fileHandle:fileHandle

		local ribProjection = s_RIBit_Option "Projection" indent:indent fileHandle:fileHandle

		if cam.orthoProjection then (
			fn motionFunc params: = 
			(
				params[1].transform = params[2].transform
				params[1].inversePOV()
				params[1].write()
			)
			ribProjection.addParameter "orthographic" undefined
			ribProjection.write()
		) else (
			ribProjection.addParameter "perspective" undefined

			

fn motionFunc params: =
			(
				local VFOV = if params[3] > 1 then (2.*atan(tan(params[2].fov/2.)/params[3])) else params[2].fov

				params[1].changeValue "fov" VFOV
				params[1].write()
			)
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribProjection, cam, renderAspect)
			ribMotion.dependsOn = #(cam.fov.isAnimated)
			ribMotion.bypass = settings.useConservativeMotion
			ribMotion.write()
			

			fn motionFunc params: = 
			(
				params[1].transform = params[2].transform
				params[1].inversePOV()
				params[1].write()
			)
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribTransform, cam)
			ribMotion.dependsOn = #(cam.transform.isAnimated)
			ribMotion.bypass = false
		)


			
		if settings.useDOF then (
			local cameraVFOV = if renderAspect > 1 then (2.*atan(tan(cam.fov/2.)/renderAspect)) else cam.fov
			local fLength = renderAspect / tan (cameraVFOV / 2.)
			local fDist = cam.baseObject.targetDistance
			Format "%DepthOfField % % %\n" indent settings.DOFAperture fLength fDist to:fileHandle
		)


		if cam.clipManually then
			Format "%Clipping % %\n" indent cam.nearClip cam.farClip to:fileHandle

		ribMotion.write()
	),
	
	
	fn exportViewport =
	(
		local viewtype = viewport.getType()


		local ribProjection = s_RIBit_Option "Projection" indent:indent fileHandle:fileHandle

		if (viewtype == #view_camera) then (
			cam = viewport.getCamera()
			exportCamera()			
		) else if (viewtype == #view_persp_user) then (
		--- code from doberman
			local renderAspect = (settings.rendWidth as float)/settings.rendHeight
			local cameraVFOV = if renderAspect > 1 then (2.*atan(tan(getViewFOV()/2.)/renderAspect)) else getViewFOV()
			local nearClip = 0.1
			local farClip = 100000.0
			if cam != undefined then
				if cam.clipManually == true then
				(
					local nearClip = cam.nearclip
					local farClip = cam.farclip
				)

			ribProjection.addParameter "perspective" undefined

			ribProjection.addParameter "fov" cameraVFOV
			ribProjection.write()

			Format "%Clipping % %\n" indent nearClip farClip to:fileHandle
			
			local ribTransform = s_RIBit_Transform (inverse (getViewTM())) indent:indent fileHandle:fileHandle
			ribTransform.inversePOV()
			ribTransform.write()

		) else if (viewtype == #view_iso_user) then (
format "RIBit::Camera::exportViewport::viewtype=%\n" viewtype
			local nearClip = 0.1
			local farClip = 100000.0
			local renderAspect = (settings.rendWidth as float)/settings.rendHeight
			local cameraVFOV = if renderAspect > 1 then (2.*atan(tan(getViewFOV()/2.)/renderAspect)) else getViewFOV()

			ribProjection.addParameter "orthographic" undefined
			ribMotion.attribute = ribProjection
			ribMotion.write()

			Format "%Clipping % %\n" indent nearClip farClip to:fileHandle
			local width = (getScreenScaleFactor (getViewTM())[4]) * tan (cameraVFOV / 2.) --/ renderAspect
			local xmin = -(2. * width) -- -(winSize) * 2. --/ renderAspect
			local xmax = 2. * width --(winSize) * 2. --/ renderAspect 
			local ymin = -(width / renderAspect) -- -(winSize) / renderAspect
			local ymax = width / renderAspect --(winSize) / renderAspect
			Format "%ScreenWindow % % % %\n" indent xmin xmax ymin ymax to:fileHandle

			local ribTransform = s_RIBit_Transform ( (getViewTM())) indent:indent fileHandle:fileHandle
--			ribTransform.transform = translate ribTransform.transform [0,0,(getScreenScaleFactor (getViewTM())[4])]
--		ribTransform.transform = inverse ribTransform.transform
--		ribTransform.transform = rotateY ribTransform.transform 180
--		ribTransform.transform *= scaleMatrix [-1,1,1]

--			ribTransform.inversePOV()
--			ribTransform.transform = rotateY ribTransform.transform 180
			ribMotion.write()
		) else (
--				format "%Projection \"orthographic\"\n" indent to:fileHandle
--				local r = mapScreenToWorldRay (getViewSize() / 2.)
--				local ribTransform = s_RIBit_Transform (matrixFromNormal r.dir) indent:indent fileHandle:fileHandle
--				ribTransform.inversePOV()
--		transform = inverse transform
--		ribTransform.transform = translate ribTransform.transform r.pos
--		ribTransform.transform = inverse ribTransform.transform 
--		ribTransform.inversePOV()
--		scale [-1,1,1]
--				ribTransform.write()

		) 
	),
	


	fn write =
	(
		if fileHandle == undefined then return()
		
		
case (superClassOf cam) of (
			-- if cam is a light, we're exporting a shadowmap RIB. 
			light: expotLightAsCamera()
			-- if cam is a camera object, use it.
			camera: exportCamera()
			-- otherwise, export the active viewport.
			default: exportViewport()
		)
		exportRIBBox()


	
)

) -- end of s_RIBit_Camera struct


