--*********************************************************************************

--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************

struct s_RIBit_Functions (


	fn replaceBackSlash str =
	(
		local outStr = copy str
		for i = 1 to str.count where str[i] == "\\" do
			outStr[i] = "/"
		outStr
	),
	

	-- Converts a frame number into integer.
	fn frameToInt f = ((f as integer) / (4800/frameRate)),


	-- joins array a2 to array a1 without duplicate items.
	fn uniqueJoin &a1 a2 =
	(
		if a2 == undefined then return()
		if not isKindOf a2 array then a2 = #(a2)
		for i in a2 where (findItem a1 i)==0 do
			append a1 i
	),

	-- converts a text of the format:"1-5,7,8,20-25" into a bitArray
	fn textToNumbers txt =
	(
		local numbers = #{}
		local numGroups = filterString txt ","
		for i in numGroups do
		(
			local g = filterString i "-"
			case g.count of  				(
				1: try(append numbers (g[1] as integer))catch()
				2: try(join numbers #{(g[1] as integer)..(g[2] as integer)})catch()
			)
		)
		numbers
	),


	-- subtracts array a2 from the array a1
	fn arraySubtract a1 a2 = (for i in a1 where (findItem a2 i == 0) collect i),


	-- checks if a character is a digit
	fn isDigit s = (try(findString "0123456789" s != undefined)catch(false)),


	-- removes spaces from a string	
	fn removeSpaces s = 
	(
		while s[s.count]==" " do s = subString s 1 (s.count-1)
		while s[1] == " " do s = subString s 2 (s.count-1)
		s
	), -- end removeSpaces fn


	-- seperates numbers from other characters in a string. returns an array of the different parts.
	fn explodeName s removeSpaces:true = 
	(
		local arr=#()
		local tmpStr = ""
		local isNum = try(isDigit s[1])catch(false)
		for i=1 to s.count do 
		(
			local d = isDigit s[i]
			if d != isNum then 
			(
				if removeSpaces then tmpStr = try(removeSpaces tmpStr)catch(tmpStr)
				if isNum then tmpStr = tmpStr as integer
				append arr tmpStr
				tmpStr = ""
				isNum = d
			)
			tmpStr += s[i]
		)

		if removeSpaces then tmpStr = try(removeSpaces tmpStr)catch(tmpStr)
		if isNum then tmpStr = tmpStr as integer
		if tmpStr!="" then append arr tmpStr
 			arr
	), -- end explodeName fn


	-- adds zeros to a number
	fn leadingZero n cnt onlyNumbers:false = 
	(
		n = n as string
		if onlyNumbers then 
		(
			local a = explodeName n removeSpaces:false
			n = ""
			for i in a do
			(
				n += (if isKindOf i Integer then (leadingZero i cnt) else i)
			)
		) else (
			while n.count < cnt do
				n = "0" + n
		)
		n
	),-- end leadingZero
	

	-- Sorts an array by the .name property. the sorting also accounts for numbers (i.e. "2" will be before "03")
	fn sortByName &items =
	(
		fn sortFn v1 v2 = 
		(
			local a1 = try(explodeName v1.name)catch(#())
			local a2 = try(explodeName v2.name)catch(#())
			local isEqual = true
			local outVal = 0
			for i = 1 to (amin a1.count a2.count) do (
				if a1[i] == a2[i] then continue
				isEqual = false
				if try(a1[i] < a2[i])catch(a1[i] as string < a2[i] as string) then outVal = -1
				else outVal = 1
				exit
			)
			if isEqual then
			(
				case of
				(
					(a1.count > a2.count): outVal = 1
					(a1.count < a2.count): outVal = -1
				)
			)
			outVal
		) --end sortFn fn
		qsort items sortFn
	), -- end sortByName fn
	
	
	fn findModifier object modClass multiple:false=
	(
		local returnVal = if multiple then #() else undefined
		for i = 1 to object.modifiers.count where (classOf object.modifiers[i] == modClass and object.modifiers[i].enabled) do (
			if not multiple then (
				returnVal = i
				exit
			) else (
				if returnVal == undefined then
					returnVal = #()
				append returnVal i
			)
		)		
		returnVal
	),
	
	
	fn floatToColor f =
	(
		f *= 255.
		color f f f
	),
	
	fn colorToFloat r g b mode:#average =
	(
		case mode of (
			#average: 	((r + g + b) / 3.) / 255.
			#red: 		r / 255.
			#green: 	g / 255.
			#blue: 		b / 255.
			#hue: 		(col = color r g b; col.h / 255.)
			#sat: 		(col = color r g b; col.s / 255.)
			#val: 		(col = color r g b; col.v / 255.)
		)
	),
	

	
fn remap inMin inMax outMin outMax val limit:false =
	(
		local p = (val - inMin) / (inMax - inMin)
		local outVal = outMin + ((outMax - outMin) * p)
		if limit == true then (
			outVal = amin outMax outVal
			outVal = amax outMin outVal
		)
		outVal
	),
	
	fn linstep inMin inMax val =
	(
		val = val as float
		local p = (val - inMin) / (inMax - inMin)
		amin 1 (amax 0 p)
	),
	
	
fn smoothstep inMin inMax val =
	(
		val = val as float
		local p = (val - inMin) / (inMax - inMin)
		p = (3 * (p ^ 2)) - (2 * (p ^ 3))
		amin 1 (amax 0 p)
	),
	
	-- returns a string in lower case
	fn toLowerCase str =
	(
		local lower = "abcdefghijklmnopqrsdtuvwxyz"
		local upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		local tmpStr = ""
		for i = 1 to str.count do
		(
			local n
			if (n=(findString upper str[i]))!= undefined then tmpStr += lower[n]
			else tmpStr += str[i]
		)
		tmpStr
	),
	

	-- returns a string in upper case
	fn toUpperCase str =
	(
		local lower = "abcdefghijklmnopqrsdtuvwxyz"
		local upper = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
		local tmpStr = ""
		for i = 1 to str.count do
		(
			local n
			if (n=(findString lower str[i]))!= undefined then tmpStr += upper[n]
			else tmpStr += str[i]
		)
		tmpStr
	),
	

	
	
	fn replaceString str findStr replaceStr =
	(
		local tmpStr = copy str
		local outStr = ""
		local i
		local f = s_RIBit_Functions()
		while (i = findString (f.toLowerCase tmpStr) (f.toLowerCase findStr)) != undefined do (
			outStr += subString tmpStr 1 (i - 1) + replaceStr as string
			tmpStr = subString tmpStr (i + findStr.count) (tmpStr.count - (i + findStr.count) + 1)
		)

		outStr += tmpStr
		outStr
	),
	
	
	

fn winToUnixPath path =
	(
		s_RIBit_Functions.replaceString (path as string) "\\" "/"
	),
	
	
	fn parseRIBBox rib execMode:false params:#() =
	(
	
	fn isEscapeChar c =
		(
			local escapeChars = "{}"
			findString escapeChars c != undefined
		)
				
		
fn parseReservedWords str params:#() =
		(
			local outStr = str

			try(
				-- variables
				outStr = s_RIBit_Functions.replaceString outStr "@F2" "(leadingZero (frameToInt currentTime) 2)"
				outStr = s_RIBit_Functions.replaceString outStr "@F3" "(leadingZero (frameToInt currentTime) 3)"
				outStr = s_RIBit_Functions.replaceString outStr "@F4" "(leadingZero (frameToInt currentTime) 4)"
				outStr = s_RIBit_Functions.replaceString outStr "@F5" "(leadingZero (frameToInt currentTime) 5)"
				outStr = s_RIBit_Functions.replaceString outStr "@F6" "(leadingZero (frameToInt currentTime) 6)"
				outStr = s_RIBit_Functions.replaceString outStr "@F" "(frameToInt currentTime)"
				try(
					outStr = s_RIBit_Functions.replaceString outStr "@obj" params[1].name
					outStr = s_RIBit_Functions.replaceString outStr "@particleCount" ((particleCount params[1]) as float)
					outStr = s_RIBit_Functions.replaceString outStr "@particleID" (params[2] as float)
					outStr = s_RIBit_Functions.replaceString outStr "@particleAge" ((s_RIBit_Functions.frameToInt (particleAge params[1] params[2])) as float)
					outStr = s_RIBit_Functions.replaceString outStr "@particleSize" (particleSize params[1] params[2])
					outStr = s_RIBit_Functions.replaceString outStr "@particlePos" (particlePos params[1] params[2])
					outStr = s_RIBit_Functions.replaceString outStr "@particleVelocity" (particleVelocity params[1] params[2])
				)catch()
				
				-- built-in functions
				outStr = s_RIBit_Functions.replaceString outStr "leadingZero" "s_RIBit_Functions.leadingZero"
				outStr = s_RIBit_Functions.replaceString outStr "frameToInt" "s_RIBit_Functions.frameToInt"
				outStr = s_RIBit_Functions.replaceString outStr "toLowerCase" "s_RIBit_Functions.toLowerCase"
				outStr = s_RIBit_Functions.replaceString outStr "toUpperCase" "s_RIBit_Functions.toUpperCase"
				outStr = s_RIBit_Functions.replaceString outStr "floatToColor" "s_RIBit_Functions.floatToColor"
	
			outStr = s_RIBit_Functions.replaceString outStr "remap" "s_RIBit_Functions.remap"
				outStr = s_RIBit_Functions.replaceString outStr "linstep" "s_RIBit_Functions.linstep"
				outStr = s_RIBit_Functions.replaceString outStr "smoothstep" "s_RIBit_Functions.smoothstep"
				outStr = s_RIBit_Functions.replaceString outStr "winToUnixPath" "s_RIBit_Functions.winToUnixPath"
			)catch()	

			outStr
		)
		
		local isEscape = false
		local outStr = ""
		local tmpStr = rib
		local i = 1
		while (i <= rib.count) do (
			if execMode and rib[i] == "}" then (
				outStr = parseReservedWords outStr params:params
				outStr = try((execute outStr) as string)catch("")
				return #(outStr, i)
			)
			isEscape = rib[i] == "\\" and i < rib.count and isEscapeChar rib[i+1]
			if isEscape then (
				i += 1
			)
			tmpStr = subString rib (i + 1) (rib.count - i)
			if not isEscape and rib[i] == "{" and (i == 1 or rib[i-1] != "#") then (
				local tmp = s_RIBit_Functions.parseRIBBox tmpStr execMode:true params:params
				if isKindOf tmp Array then (
					outStr += tmp[1]
					i += tmp[2]
				)
			) else (
				outStr += rib[i]
			)
			i += 1
		)
		outStr
	),
	
	
	fn writeValueByType val brackets:false =
	(
		local s = "" as stringStream
		case (classOf val) of (
			Float: 		if brackets then format " [%]" val to:s else format " %" val to:s
			Integer: 	if brackets then format " [%]" val to:s else format " %" val to:s
			String: 	if brackets then format " [\"%\"]" val to:s else format " \"%\"" val to:s
			Time: 		if brackets then format " [%]" (s_RIBit_Functions.frameToInt val) to:s else format " %" (s_RIBit_Functions.frameToInt val) to:s
			Array: (
				local values = ""
				for i = 1 to val.count do
					values += s_RIBit_Functions.writeValueByType val[i] + (if i < val.count then " " else "")
--					values += val[i] as string + (if i < val.count then " " else "")
				format " [%]" values to:s
			)
			Color:		if brackets then format " [% % %]" (val.r / 255.) (val.g / 255.) (val.b / 255.) to:s 
							else format " % % %" (val.r / 255.) (val.g / 255.) (val.b / 255.) to:s
			s_RIBit_Parameter: format "%" (val.write()) to:s
		)
		s as string
	)


	
)-- end of s_RIBit_Functions struct
