--*********************************************************************************
--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************

struct s_RIBit_Geometry (
	object,
	shadersToComplie = #(),
	shadersDir = "",


	force2sided = false,
	camObject,
	settings = s_RIBit_Settings(),
	indent = "",
	fileHandle,
	
	
	fn exportPolygons params: =
	(
		local type = params[1]
		local object = params[2]
		local indent = params[3]
		local fileHandle = params[4]
		max create mode
		disableSceneRedraw()
		------------------
		-- check whether it is a poly
		-----------------
		local obj = snapshot object
		convertToPoly obj
		local facecount = getNumFaces obj
		
		case type of (
			1: format "%PointsPolygons \n" indent to:fileHandle
			2: format "%SubdivisionMesh \"catmull-clark\"\n" indent to:fileHandle
		)
		
		--if ((mode == 1) or (mode == 2)) then(
		--First output the number of vertices per face
		format "%\t[" indent to:fileHandle
		for i=1 to facecount do(
			if (mod i 25 == 0 )then
				format"\n%\t" indent to:fileHandle
			local faceverts = polyop.getFaceVerts obj i
			local numverts = faceverts.count
			format " % " numverts to:fileHandle
		)
		format "]\n" to:fileHandle
		
		-- now output the vertex indexes
		format "%\t[" indent to:fileHandle
		for i=1 to facecount do(
			if (mod i 9 == 0 ) then
				format"\n%\t" indent to:fileHandle
			local faceverts = polyop.getFaceVerts obj i
			for j in faceverts do(
				format " % " (j - 1) to:fileHandle
			)
		)
		format "]\n" to:fileHandle
		
		-- now output the position of each vertex
		format "%\t\"P\" [" indent to:fileHandle
		local numverts = getNumVerts obj
		for i = 1 to numverts do (
			if (mod i 9 == 0 )then
				format"\n%\t" indent to:fileHandle
			vertpos = in coordsys local polyop.getVert obj i
			format " % % % " vertpos[1] vertpos[2] vertpos[3] to:fileHandle
		)
		format "]\n" to:fileHandle



		--output vertex channels
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_VertexAttribute multiple:true
		for i in modIndex do (
			local chan = object.modifiers[i].Channel
			local chanSupported = false
			case chan of (
				1: (chan = 0; chanSupported = polyOp.getMapSupport obj chan)
				2: (chan = 3; chanSupported = polyOp.getVDataChannelSupport obj chan)
			)
			if chanSupported then (
				local attrType = case object.modifiers[i].AttributeType of (
						1: "float"
						2: "color"
					)
				format "%\t\"vertex % %\" [" indent attrType object.modifiers[i].AttributeName to:fileHandle
				for v = 1 to numverts do (
					if (mod v 9 == 0 )then
						format"\n%\t" indent to:fileHandle
					case chan of (
						0: ( -- vertex color
							local col = polyOp.getMapVert obj 0 v
							case object.modifiers[i].AttributeType of (
								1: (col *= 255.; format " % " (s_RIBit_Functions.colorToFloat col[1] col[2] col[3] mode:#average) to:fileHandle)
								2: format " % % % " col[1] col[2] col[3] to:fileHandle
							)
						)
						default: ( -- vertex VData Channel
							local val = (polyOp.getVDataValue obj chan v)
							case object.modifiers[i].AttributeType of (
								1: format " % " val to:fileHandle
								2: format " % % % " val val val to:fileHandle
							)
							format " % "  to:fileHandle
						)
					)
				)
				format "]\n" to:fileHandle
			)
		)
		

		-- now output the normal of each vertex
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Geometry
		if modIndex == undefined or object.modifiers[modIndex].ExportNormals then (
			format "%\t\"N\" [" indent to:fileHandle
			convertToMesh obj
			local numverts = getNumVerts obj
			for i = 1 to numverts do (
				if (mod i 9 == 0 )then
					format"\n%\t" indent to:fileHandle
				normpos = in coordsys object getNormal obj i
				format " % % % " normpos[1] normpos[2] normpos[3] to:fileHandle
			)
			format "]\n" to:fileHandle
		)


		delete obj
		enablesceneredraw()
	),


	fn exportRenderingAttributes =
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Shaders
		if modIndex == undefined then (
			local objcolor = object.wirecolor/255.
			format "%Color % % %\n" indent objcolor.r objcolor.g objcolor.b to:fileHandle
			format "%Opacity % % %\n" indent objcolor.a objcolor.a objcolor.a to:fileHandle
			format "%Surface \"matte\"\n" indent to:fileHandle
		) else (
			local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle

		-- color
			fn motionFunc params: = 
			(
				params[1].params = #()
				params[1].addParameter (getProperty params[2] params[3]) undefined
				params[1].write()
			)

			local ribColor = s_RIBit_Option "Color" indent:indent fileHandle:fileHandle
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribColor, object.modifiers[modIndex], "Color")

			ribMotion.dependsOn = #(object.modifiers[modIndex].color.isAnimated)
			ribMotion.bypass = settings.useConservativeMotion
			ribMotion.write()
		-- Opacity
			ribColor.name = "Opacity"
			ribMotion.funcParams = #(ribColor, object.modifiers[modIndex], "Opacity")

			ribMotion.dependsOn = #(object.modifiers[modIndex].opacity.isAnimated)
			ribMotion.write()

		-- surface
			local ribShader = s_RIBit_Option "Surface" indent:indent fileHandle:fileHandle
			fn motionFunc params: =
			(
				params[1].params = #()
				params[1].addParameter params[2] undefined
				params[1].paramsFromText (s_RIBit_Functions.parseRIBBox params[3] params:#(params[4]))
				params[1].write()
			)
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribShader, object.modifiers[modIndex].Surface, object.modifiers[modIndex].SurfaceParams, object)
			ribMotion.dependsOn = #(object.modifiers[modIndex].SurfaceParams != "")
			ribMotion.write()
			if object.modifiers[modIndex].Surface != "" then (
				local shaderSource = shadersDir + object.modifiers[modIndex].Surface
				if doesFileExist (shaderSource + ".sl") then
					append shadersToComplie #((shaderSource + ".sl"), shaderSource)
			)
				
			if object.modifiers[modIndex].Displacement != "" then (
				ribShader.name = "Displacement"
	
			ribMotion.funcParams = #(ribShader, object.modifiers[modIndex].Displacement, object.modifiers[modIndex].DisplacementParams, object)
				ribMotion.dependsOn = #(object.modifiers[modIndex].DisplacementParams != "")
				ribMotion.write()
				local shaderSource = shadersDir + object.modifiers[modIndex].Displacement
				if doesFileExist (shaderSource + ".sl") then
					append shadersToComplie #((shaderSource + ".sl"), shaderSource)
			)
			if object.modifiers[modIndex].Atmosphere != "" then(
				ribShader.name = "Atmosphere"
				ribMotion.funcParams = #(ribShader, object.modifiers[modIndex].Atmosphere, object.modifiers[modIndex].AtmosphereParams, object)
				ribMotion.dependsOn = #(object.modifiers[modIndex].AtmosphereParams!= "")
				ribMotion.write()
				local shaderSource = shadersDir + object.modifiers[modIndex].Atmosphere
				if doesFileExist (shaderSource + ".sl") then
					append shadersToComplie #((shaderSource + ".sl"), shaderSource)
			)
			format "%ShadingInterpolation \"%\"\n" indent object.modifiers[modIndex].ShadingInterpolation to:fileHandle
			format "%Matte %\n" indent (if object.modifiers[modIndex].Matte then 1 else 0) to:fileHandle
		)
	),
	
	

	fn exportGeometry =
	(
		-- Export Polygons

		local mode = 1
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Geometry
		if modIndex == undefined then 
			mode = 1
		else (
			if (object.modifiers[modIndex].ExportPoly and not object.modifiers[modIndex].ExportSubd) then
				mode = 1
			if (object.modifiers[modIndex].ExportPoly and object.modifiers[modIndex].ExportSubd) then
				mode = 2
		)

		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		ribMotion.func = exportPolygons
		ribMotion.funcParams = #(mode, object, indent, fileHandle)
		ribMotion.dependsOn = #(true)
		ribMotion.write()

	),
	
	
	
fn exportParticles =
	(
	
	local ribParticle = s_RIBit_Particle object:object camObject:camObject settings:settings indent:indent fileHandle:fileHandle
		ribParticle.write()
	),
	
	
	fn exportLightLink =
	(
		local allLights = for l in lights where (not isKindOf l Targetobject) collect l
		for i = 1 to allLights.count do (
			local l = allLights[i]
			local ribIllum = s_RIBit_Option "Illuminate" indent:indent fileHandle:fileHandle
			ribIllum.addParameter i undefined

			if (l.includeList != undefined) and (findItem l.includeList object > 0) then (
				ribIllum.addParameter 1 undefined
				ribIllum.write()
			)

			if (l.excludeList != undefined) and (findItem l.excludeList object > 0) then (
				ribIllum.addParameter 0 undefined
				ribIllum.write()
			)
		)
	),
	

	fn exportRIBBox =
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_RIBBox multiple:true
		for i in modIndex do (
			format "%##---- RIB Box ----##\n" indent to:fileHandle
			local lines = filterString (s_RIBit_Functions.parseRIBBox object.modifiers[i].RIBCode params:#(object)) "\n\r"
			for l in lines do
				format "%%\n" indent l to:fileHandle
		)
	),

	fn write =
	(
		if fileHandle == undefined then return()
		if (isKindOf object GeometryClass) and not (isKindOf object targetObject) and object.renderable and not object.isHidden then (
			-- Rendering Attributes
			exportRenderingAttributes()
			exportLightLink()
	
			local ribTransform = s_RIBit_Transform concat:true indent:indent fileHandle:fileHandle
			ribTransform.transformBegin()
			indent = ribTransform.indent

			local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
			ribMotion.func = fn motionFunc params: = (params[1].transform = params[2].transform; params[1].write())
			ribMotion.funcParams = #(ribTransform, object)
			ribMotion.dependsOn = #(object.transform.isAnimated)
			ribMotion.write()
	
			exportRIBBox()
	

			case (classOf object) of (
				Snow: 		exportParticles()
				Spray: 		exportParticles()
				Blizzard: 	exportParticles()
				PArray: 	exportParticles()
				PCloud: 	exportParticles()
				SuperSpray: exportParticles()
				default: 	exportGeometry()
			)

			ribTransform.transformEnd()
			indent = ribTransform.indent
			-- End Transformation block
		) 
	)

) -- end of s_RIBit_Geometry struct













