--*********************************************************************************
--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************


struct s_RIBit_Shadowmap (
	object, 
	name = "", 
	RIBFile = ""
) -- end of s_RIBit_Shadowmap struct


struct s_RIBit_Light (
	light,
	lightID = 1,

	shadersToComplie = #(),
	shadowmapName = "",
	shadersDir = "",
	shadowSamples = 1.0,
	indent = "",
	fileHandle,
	settings = s_RIBit_Settings(),


	fn exportRIBBox =
	(
		local modIndex = s_RIBit_Functions.findModifier light mo_RIBit_RIBBox multiple:true
		for i in modIndex do (
			format "%##---- RIB Box ----##\n" indent to:fileHandle
			local lines = filterString (s_RIBit_Functions.parseRIBBox light.modifiers[i].RIBCode params:#(light)) "\n\r"
			for l in lines do
				format "%%\n" indent l to:fileHandle
		)
	),
 

	fn writeAmbient =
	(
		local ribLight = s_RIBit_Option "LightSource" indent:indent fileHandle:fileHandle
		ribLight.addParameter "ambientlight" undefined
		ribLight.addParameter lightID undefined
		ribLight.addParameter "intensity" 1.0
		ribLight.addParameter "lightcolor" light
		
		ribLight.write()
	),



	fn writeUberLight params: = --type distant:false =
	(
	
	local light = params[1]
		local lightID = params[2]
		local shadowmapName = params[3]
		local shadersDir = params[4]
		local shadowSamples = params[5]
		local indent = params[6]
		local fileHandle = params[7]
		local type = params[8]
		local distant = if params[9] == true then true else false
	
		local shaderSource = getDir #scripts + "\\RIBit\\Shaders\\uberlight.sl"
		local shaderFile = "uberlight"
--		append shadersToComplie #((getDir #scripts + "\\RIBit\\Shaders\\uberlight.sl"), (shadersDir + "uberlight"))

		local ribLight = s_RIBit_Option "LightSource" indent:indent fileHandle:fileHandle
		ribLight.addParameter shaderFile undefined
		ribLight.addParameter lightID undefined
		ribLight.addParameter "intensity" light.multiplier
		ribLight.addParameter "lightcolor" light.rgb
		ribLight.addParameter "string lighttype" type

		if light.useNearAtten then (
			ribLight.addParameter "float cuton" light.nearAttenEnd
			ribLight.addParameter "float nearedge" (light.nearAttenEnd - light.nearAttenStart)
		)

		if light.useFarAtten then (
			ribLight.addParameter "float cutoff" light.farAttenStart
			ribLight.addParameter "loat faredge" (light.farAttenEnd - light.farAttenStart)
		)

		ribLight.addParameter "float falloff" (light.attenDecay - 1) --0: none; 1:Inverse Linear; 2:Inverse Squaew;
		ribLight.addParameter "float falloffdist" light.decayRadius
		ribLight.addParameter "float maxintensity" light.multiplier
		ribLight.addParameter "float parallelrays" (if distant then 1 else 0)

		if type == "spot" then (
			local aspect = light.aspect
			local width =  tan (light.hotspot / 2.) * sqrt aspect
			local height = tan (light.hotspot / 2.) / sqrt aspect
			local wedge = tan (light.falloff / 2.) * sqrt aspect - width
			local hedge = tan (light.falloff / 2.) / sqrt aspect - height
			ribLight.addParameter "float width" width
			ribLight.addParameter "float height" height
			ribLight.addParameter "float wedge" wedge
			ribLight.addParameter "float hedge" hedge
			ribLight.addParameter "float roundness" (2 - light.coneShape)
		) else if type == "distant" then (
			local aspect = light.aspect
			local width =  light.hotspot
			local height = light.hotspot / aspect
			local wedge = light.falloff - width
			local hedge = (light.falloff / aspect) - height
			ribLight.addParameter "float width" width
			ribLight.addParameter "float height" height
			ribLight.addParameter "float wedge" wedge
			ribLight.addParameter "float hedge" hedge
			ribLight.addParameter "float roundness" (2 - light.coneShape)	
		)
		ribLight.addParameter "float beamdistribution" 0

		-- shodows


		if (light.baseObject.castShadows) then (
			ribLight.addParameter "color shadowcolor" light.shadowColor
			if light.baseObject.raytracedShadows then (
				ribLight.addParameter "string shadowmap" "raytrace"
				
				local ribShadow = s_RIBit_Attribute "visibility" indent:indent fileHandle:fileHandle
				ribShadow.addParameter "transmission" "opaque"
			) else if shadowmapName != "" then (
					ribLight.addParameter "string shadowmap" shadowmapName
					ribLight.addParameter "float shadownsamps" shadowSamples
					ribLight.addParameter "float shadowblur" (light.samplerange / 1000.)
					ribLight.addParameter "float shadowbias" light.mapbias
			) 
		)


		-- advanced effects: don't affect diffuse/specular. NOTE: this requires that the surface shader be able to handle this.
		if (not light.affectDiffuse or light.ambientOnly) then
			ribLight.addParameter "float __nondiffuse" 1

		if (not light.affectSpecular or light.ambientOnly) then
			ribLight.addParameter "float __nonspecular" 1

		
		ribLight.write()



		if ribShadow != undefined then ribShadow.write()
	),
	
	fn motionUberLight ribMotion =
	(
			append shadersToComplie #((getDir #scripts + "\\RIBit\\Shaders\\uberlight.sl"), (shadersDir + "uberlight"))
			ribMotion.write()
	),

	fn write =
	(
		if (classOf light) == Color then 
			writeAmbient()
		else (
			local ribTransform = s_RIBit_Transform light.transform concat:true indent:indent fileHandle:fileHandle
			ribTransform.transformBegin()

			indent = ribTransform.indent

		
			local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
			fn motionFunc params: =
			(
				params[1].transform = params[2].transform
				params[1].localRotateY 180
				params[1].write()
			)
			ribMotion.func = motionFunc
			ribMotion.funcParams = #(ribTransform, light)
			ribMotion.dependsOn = #(light.transform.isAnimated)
			ribMotion.write()


			ribMotion.func = writeUberLight
			ribMotion.dependsOn = #(light.isAnimated)
			ribMotion.bypass = settings.useConservativeMotion
			ribMotion.funcParams = #(light, lightID, shadowmapName, shadersDir, shadowSamples, indent, fileHandle)
			case (classOf light) of (
				Omnilight: 				(join ribMotion.funcParams #("omni", false); motionUberLight ribMotion)
				TargetDirectionallight: (join ribMotion.funcParams #("distant",true); motionUberLight ribMotion)
				Directionallight: 		(join ribMotion.funcParams #("distant",true); motionUberLight ribMotion)
				targetSpot: 			(join ribMotion.funcParams #("spot"
, false); motionUberLight ribMotion)
				freeSpot: 				(join ribMotion.funcParams #("spot", false); motionUberLight ribMotion)
			)


/*
			case (classOf light) of (
				Omnilight: writeUberLight "omni"
				TargetDirectionallight: writeUberLight "distant" distant:true
				Directionallight: writeUberLight "distant" distant:true
				targetSpot: writeUberLight "spot"
				freeSpot: writeUberLight "spot"
			)
*/					


			
if light.includeList != undefined then (
				local ribIllum = s_RIBit_Option "Illuminate" indent:indent fileHandle:fileHandle
				ribIllum.addParameter lightID undefined
				ribIllum.addParameter 0 undefined
				ribIllum.write()
			)				
			
			exportRIBBox()
			ribTransform.transformEnd()
			indent = ribTransform.indent
		)
	)

) -- end of s_RIBit_Light struct




/*
A.3 Light Source Shaders

A.3.1 Ambient light source
light
ambientlight(
	float intensity = 1;
	color lightcolor = 1;
)
{
	Cl = intensity * lightcolor;
}

A.3.2 Distant light source
light
distantlight(
	float intensity = 1;
	color lightcolor = 1;
	point from = point "shader" (0,0,0);
	point to = point "shader" (0,0,1);
)
{
	solar(to-from, 0.0)
	Cl = intensity * lightcolor;
}

A.3.3 Point light source
light
pointlight(
	float intensity = 1;
	color lightcolor = 1;
	point from = point "shader" (0,0,0);
)
{
	illuminate(from)
	Cl = intensity * lightcolor / L.L;
}

A.3.4 Spotlight source
light
spotlight(
	float intensity = 1;
	color lightcolor = 1;
	point from = point "shader" (0,0,0);
	point to = point "shader" (0,0,1);
	float coneangle = radians(30);
	float conedeltaangle = radians(5);
	float beamdistribution = 2;
)
{
	float atten, cosangle;
	uniform vector A = (to - from)/length(to-from);
	illuminate(from, A, coneangle) {
		cosangle = L . A / length(L);
		atten = pow(cosangle, beamdistribution) / L.L;
		atten *= smoothstep(cos(coneangle),
		cos(coneangle - conedeltaangle), cosangle);
		Cl = atten * intensity * lightcolor;
	}
}


*/