--*********************************************************************************
--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes
--*********************************************************************************

struct s_RIBit_Particle (

	object,
	activeParticles = #{},
	camObject,
	settings = s_RIBit_Settings(),
	indent = "",
	fileHandle,
	
	
	fn incIndent = 
	(
		indent += "\t"
	),
	
	fn decIndent =
	(
		if indent.count > 0 then
			indent = subString indent 1 (indent.count - 1)
	),

	fn getActiveParticles =
	(
		activeParticles = #{}
		for i = 1 to particleCount object do
			activeParticles[i] = particleAge object i != undefined
	),



	fn exportRIBBox =
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_RIBBox multiple:true
		if modIndex != undefined then (
			format "%##---- RIB Box ----##\n" indent to:fileHandle
			for i in modIndex do (
				local lines = filterString (s_RIBit_Functions.parseRIBBox object.modifiers[i].RIBCode params:#(object)) "\n\r"
				for l in lines do
					format "%%\n" indent l to:fileHandle
			)
		)
	),


	fn getParticleColor i obj: =
	(
		if obj == unsupplied then obj = object
		local col = white
		local modIndex = s_RIBit_Functions.findModifier obj mo_RIBit_Particles
		if modIndex != undefined and obj.modifiers[modIndex].colorPP != "" then (
			col = try(execute (s_RIBit_Functions.parseRIBBox obj.modifiers[modIndex].colorPP params:#(obj, i)))catch(white)
		) else if (modIndex = s_RIBit_Functions.findModifier obj mo_RIBit_Shaders) != undefined then (
			col = obj.modifiers[modIndex].Color
		)
		col
	),

	fn getParticleOpacity i obj: =
	(
		if obj == unsupplied then obj = object
		local op = white
		local modIndex = s_RIBit_Functions.findModifier obj mo_RIBit_Particles
		if modIndex != undefined and obj.modifiers[modIndex].opacityPP != "" then (
			op = try(execute (s_RIBit_Functions.parseRIBBox obj.modifiers[modIndex].opacityPP params:#(obj, i)))catch(white)
		) else if (modIndex = s_RIBit_Functions.findModifier obj mo_RIBit_Shaders) != undefined then (
			op = obj.modifiers[modIndex].Opacity
		)
		op
	),

	fn getParticleSize i obj: =
	(
		if obj == unsupplied then obj = object
		local s = 1.0
		local modIndex = s_RIBit_Functions.findModifier obj mo_RIBit_Particles
		if modIndex != undefined and obj.modifiers[modIndex].sizePP != "" then (
			s = s_RIBit_Functions.parseRIBBox obj.modifiers[modIndex].sizePP params:#(obj, i)
		) else 
			s = particleSize obj i

		s
	),
	
	
	
fn exportPoints params: = 
	(
		local indent = params[1]
		local fileHandle = params[2]
		local object = params[3]
		local activeParticles = params[4]
		local getParticleSize = params[5]
		local getParticleColor = params[6]
		local getParticleOpacity = params[7]
	
		format "%Points\n" indent to:fileHandle

		indent += "\t"
		local ii = 1
		
		-- export positions
		format "%\"P\" [" indent to:fileHandle
		for i in activeParticles do (
			if ii > 8 then (

				format "\n%" indent to:fileHandle
				ii = 1
			)
			local p = in CoordSys object particlePos object i
			format " % % % " p.x p.y p.z to:fileHandle
			ii += 1
		)
		format "]\n" indent to:fileHandle
		
		-- export widths
		ii = 1
		format "%\"width\" [" indent to:fileHandle
		for i in activeParticles do (
			if ii > 24 then (
				format "\n%" indent to:fileHandle
				ii = 1
			)
			format " %" (getParticleSize i obj:object) to:fileHandle
			ii += 1
		)
		format "]\n" indent to:fileHandle

		-- export colors
		ii = 1
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Particles
		if modIndex != undefined and object.modifiers[modIndex].colorPP != "" then (
			format "%\"Cs\" [" indent to:fileHandle
			for i in activeParticles do (
				if ii > 8 then (
					format "\n%" indent to:fileHandle
					ii = 1
				)
				local Cs = (getParticleColor i obj:object)
				format " % % %" (Cs.r / 255.) (Cs.g / 255.) (Cs.b / 255.) to:fileHandle
				ii += 1
			)
			format "]\n" indent to:fileHandle
		)
	
		-- export opacities
		ii = 1
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Particles
		if modIndex != undefined and object.modifiers[modIndex].opacityPP != "" then (
			format "%\"Os\" [" indent to:fileHandle
			for i in activeParticles do (
				if ii > 8 then (
					format "\n%" indent to:fileHandle
					ii = 1
				)
				local Os = (getParticleOpacity i obj:object)
				format " % % % " (Os.r / 255.) (Os.g / 255.) (Os.b / 255.) to:fileHandle
				ii += 1
			)
			format "]\n" indent to:fileHandle
		)

		-- export ages
		ii = 1
		format "%\"varying float age\" [" indent to:fileHandle
		for i in activeParticles do (
			if ii > 24 then (
				format "\n%" indent to:fileHandle
				ii = 1
			)
			local age = particleAge object i
			format " %" (s_RIBit_Functions.frameToInt age) to:fileHandle
			ii += 1
		)
		format "]\n" indent to:fileHandle
	),


	fn exportMotionPoints =
	(
		if (s_RIBit_Functions.findModifier object mo_RIBit_Shaders) == undefined then (
			local ribSurface = s_RIBit_Option "Surface" indent:indent fileHandle:fileHandle
			ribSurface.addParameter "constant" undefined
			ribSurface.write()
		)
		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		ribMotion.func = exportPoints
		ribMotion.funcParams = #(indent, fileHandle, object, activeParticles, getParticleSize, getParticleColor, getParticleOpacity)
		ribMotion.dependsOn = #(true)
		ribMotion.write()
	),


	fn exportSpheres = 
	(
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Particles

		local mat = matrix3 1
		local ribTransform = s_RIBit_Transform mat concat:true indent:indent fileHandle:fileHandle

		-- define the motion struct for transfrom
		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn motionFunc params: =
		(
			params[4].row4 = in CoordSys world particlePos params[2] params[3]
			params[1].transform = params[4] * (inverse params[2].transform)
			params[1].write()	
		)

		ribMotion.func = motionFunc
		ribMotion.dependsOn = #(true)

		-- define the motion structs for color and opacity

		local ribColorMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn colorMotionFunc params: =
		(
			params[1].params = #()
			params[1].addParameter (params[4] params[3] obj:params[2]) undefined
			params[1].write()
		)
		ribColorMotion.func = colorMotionFunc
		ribColorMotion.dependsOn = #(true)
		ribColorMotion.bypass = settings.useConservativeMotion

		local ribColor = s_RIBit_Option "Color" indent:indent fileHandle:fileHandle

		local ribOpacity = s_RIBit_Option "Opacity" indent:indent fileHandle:fileHandle


		-- define the motion structs for the sphere object
		local ribSphereMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn sphereMotionFunc params: =
		(
			params[1].params = #()
			local r = try((params[4] i obj:params[2]) as float)catch(particleSize params[2] params[3])
			params[1].addParameter r undefined
			params[1].addParameter -r undefined
			params[1].addParameter r undefined
			params[1].addParameter 360 undefined
			params[1].addParameter "uniform float age" (particleAge params[2] params[3])
			params[1].write()
		)

		ribSphereMotion.func = sphereMotionFunc
		ribSphereMotion.dependsOn = #(true)


		local ribSphere = s_RIBit_Option "Sphere" indent:indent fileHandle:fileHandle


		-- loop through the particles and export the data
		for i in activeParticles do (
			ribTransform.transformBegin()
			indent = ribTransform.indent

			ribMotion.funcParams = #(ribTransform, object, i, mat)
			ribMotion.indent = indent
			ribMotion.write()


			if modIndex != undefined and object.modifiers[modIndex].colorPP != "" then (
				ribColorMotion.funcParams = #(ribColor, object, i, getParticleColor)
				ribColor.indent = ribColorMotion.indent = indent
				ribColorMotion.write()
			)
			if modIndex != undefined and object.modifiers[modIndex].opacityPP != "" then (
				ribColorMotion.funcParams = #(ribOpacity, object, i, getParticleOpacity)
				ribOpacity.indent = ribColorMotion.indent = indent
				ribColorMotion.write()
			)


			if modIndex != undefined and object.modifiers[modIndex].RIBBoxPP != "" then (
				local lines = filterString (s_RIBit_Functions.parseRIBBox object.modifiers[modIndex].RIBCode params:#(object, i)) "\n\r"
				for l in lines do
					format "%%\n" indent l to:fileHandle
			)

			ribSphereMotion.funcParams = #(ribSphere, object, i, getParticleSize)
			ribSphere.indent = ribSphereMotion.indent = indent
			ribSphereMotion.write()
			
			ribTransform.transformEnd()
			indent = ribTransform.indent
			-- End Transformation block

		)
	),
	
	
	fn exportSprites = 
	(
		local camDir = if isValidNode camObject then camObject.dir else (inverse(getViewTM())).row3
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Particles
		local ribTransform = s_RIBit_Transform concat:true indent:indent fileHandle:fileHandle

		-- calculate particle transformation so it has the same direction as the camera
		local Zvect = normalize (-camDir)
		local Xvect = normalize (cross [0,0,1] Zvect)
		local Yvect = normalize (cross Xvect Zvect)
		local mat = matrix3 -Xvect Yvect Zvect [0,0,0]


		-- define the motion struct for transfrom
		local ribMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn motionFunc params: =
		(
			params[4].row4 = in CoordSys world particlePos params[2] params[3]
			params[1].transform = params[4] * (inverse params[2].transform)
			params[1].write()	
		)

		ribMotion.func = motionFunc
		ribMotion.dependsOn = #(true)

		-- define the motion structs for color and opacity

		local ribColorMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn colorMotionFunc params: =
		(
			params[1].params = #()
			params[1].addParameter (params[4] params[3] obj:params[2]) undefined
			params[1].write()
		)
		ribColorMotion.func = colorMotionFunc
		ribColorMotion.dependsOn = #(true)
		ribColorMotion.bypass = settings.useConservativeMotion

		local ribColor = s_RIBit_Option "Color" indent:indent fileHandle:fileHandle
		local ribOpacity = s_RIBit_Option "Opacity" indent:indent fileHandle:fileHandle
		

		-- define the motion structs for the sprite patch
		local ribSpriteMotion = s_RIBit_Motion settings:settings indent:indent fileHandle:fileHandle
		fn spriteMotionFunc params: =
		(
			params[1].params = #()
			params[1].addParameter "bilinear" undefined
			local r = try((params[4] params[3] obj:params[2]) as float) catch((particleSize params[2] params[3]))
			local x = r / 2.
			local y = r / 2.
			params[1].addParameter "P" #(-x,-y,0, x,-y,0, -x,y,0, x,y,0)
			params[1].addParameter "uniform float age" (particleAge params[2] params[3])
			params[1].write()
		)
		ribSpriteMotion.func = spriteMotionFunc
		ribSpriteMotion.dependsOn = #(true)

		local ribSprite = s_RIBit_Option "Patch" indent:indent fileHandle:fileHandle


	-- loop through the particles and export the data
		for i in activeParticles do (
			ribTransform.transformBegin()
			indent = ribTransform.indent


			ribMotion.funcParams = #(ribTransform, object, i, mat)
			ribTransform.indent = ribMotion.indent = indent
			ribMotion.write()

			if modIndex != undefined and object.modifiers[modIndex].colorPP != "" then (
				ribColorMotion.funcParams = #(ribColor, object, i, getParticleColor)
				ribColor.indent = ribColorMotion.indent = indent
				ribColorMotion.write()
			)
			if modIndex != undefined and object.modifiers[modIndex].opacityPP != "" then (
				ribColorMotion.funcParams = #(ribOpacity, object, i, getParticleOpacity)
				ribOpacity.indent = ribColorMotion.indent = indent
				ribColorMotion.write()
			)

			if modIndex != undefined and object.modifiers[modIndex].RIBBoxPP != "" then (
				local lines = filterString (s_RIBit_Functions.parseRIBBox object.modifiers[modIndex].RIBCode params:#(object, i)) "\n\r"
				for l in lines do
					format "%%\n" indent l to:fileHandle
			)

			ribSpriteMotion.funcParams = #(ribSprite, object, i, getParticleSize)
			ribSprite.indent = ribSpriteMotion.indent = indent
			ribSpriteMotion.write()

			ribTransform.transformEnd()
			indent = ribTransform.indent
			-- End Transformation block

		)
	),


	fn write =
	(
		if fileHandle == undefined then return()

	
	getActiveParticles()
	
		local modIndex = s_RIBit_Functions.findModifier object mo_RIBit_Particles
		if modIndex != undefined then (
			case object.modifiers[modIndex].type of (
				1: exportMotionPoints()
--				2: exportCurves()
				3: exportSpheres()
--				4: exportBlobbies()
				5: exportSprites()
				
default: exportMotionPoints()
			)
		) else 
			exportMotionPoints()
	)
) -- end of s_RIBit_Particle struct


/*

particleSize (const NodeGeneric): particleSize()
particleAge (const NodeGeneric): particleAge()
particleVelocity (const NodeGeneric): particleVelocity()
particlePos (const NodeGeneric): particlePos()
particleCount (const NodeGeneric): particleCount()

*/