--*********************************************************************************

--* This file is part of the RIBit Renderman(tm) exporter for max
--* By Ofer Zelichover
--* 2004/3/1 (c)
--* based in part on kcman by Rudy Cortes

--*********************************************************************************


struct s_RIBit_Renderer (
	name,
	path,
	cmdLineParams,
	slCompiler,
	compiledSLExt
) -- end of s_RIBit_Renderer struct

struct s_RIBit_Settings (
	INIFile = (getDir #scripts + "\\RIBit\\" + "RIBit_Settings.ini"),
	-- files, directories and apps
	renderers = #(),
	RIBDir = sysInfo.tempdir,
	ShaderDir = sysInfo.tempdir,
	TextureDir = sysInfo.tempdir,
	ImageDir = sysInfo.tempdir,
	
	-- render settings
	frame = 0,
	rendWidth = 640,
	rendHeight = 480,
	rendPixelAspect = 1.0,
	dispName = "unnamed",
	dispType = "framebuffer",
	dispChannels = "rgb",
	filter = #("catmull-rom", 2, 2),
	pixelSamples = #(2, 2),
	shadingRate = 1,
	renderOptions = #(),
	activeRenderer = s_RIBit_Renderer,
	useDOF = false,
	DOFAperture = 5.6,
	useMotionBlur = false,
	ShutterAngle = 90,
	useConservativeMotion = true,
	subFrames = 2,
	

	
	
fn validateFilename = (doesFileExist INIFile),
	
	
fn addRenderer name path cmdLineParams slCompiler compiledSLExt =
	(
		append renderers (s_RIBit_Renderer name path cmdLineParams slCompiler compiledSLExt)
	),
	
	
fn removeRenderer index =
	(
		if not (isKindOf index Integer) or index < 1 or index > renderers.count then return()
		deleteItem renderers index
	),
	
	fn readRenderersList =
	(
		if not (validateFilename()) then return false
		local i = 1
		local str = ""
		while (str = (getINISetting INIFile "Renderers" ("Renderer" + i as string))) != "" do (

			local arr = filterString str "\t"

			if arr.count < 5 then
				arr[5] = undefined
			for ii = 1 to arr.count where arr[ii] == undefined do arr[ii] = ""
			addRenderer arr[1] arr[2] arr[3] arr[4] arr[5]
			i += 1
		)
		true
	),
	
	fn writeRenderersList =
	(
		for i = 1 to renderers.count do
			setINISetting INIFile "Renderers" ("Renderer" + i as string) (renderers[i].name + "\t" + renderers[i].path + "\t" + renderers[i].cmdLineParams \
												+ "\t" + renderers[i].slCompiler + "\t" + renderers[i].compiledSLExt)
	),
	
	fn readDirectories =
	(
		RIBDir = getINISetting INIFile "Directories" "RIBDir"
		if RIBDir == "" then
			RIBDir = sysInfo.tempdir
		if RIBDir[RIBDir.count] != "\\" and RIBDir[RIBDir.count] != "/" then
			RIBDir += "\\"
		ShaderDir = getINISetting INIFile "Directories" "ShaderDir"
		if ShaderDir == "" then
			ShaderDir = sysInfo.tempdir
		if ShaderDir[ShaderDir.count] != "\\" and ShaderDir[ShaderDir.count] != "/" then
			ShaderDir += "\\"
		TextureDir = getINISetting INIFile "Directories" "TextureDir"
		if TextureDir == "" then
			TextureDir = sysInfo.tempdir
		if TextureDir[TextureDir.count] != "\\" and TextureDir[TextureDir.count] != "/" then
			TextureDir += "\\"
		ImageDir = getINISetting INIFile "Directories" "ImageDir"
		if ImageDir == "" then
			ImageDir = sysInfo.tempdir
		if ImageDir[ImageDir.count] != "\\" and ImageDir[ImageDir.count] != "/" then
			ImageDir += "\\"
	),
	
	fn writeDirectories =
	(
		setINISetting INIFile "Directories" "RIBDir" RIBDir
		setINISetting INIFile "Directories" "ShaderDir" ShaderDir
		setINISetting INIFile "Directories" "TextureDir" TextureDir
		setINISetting INIFile "Directories" "ImageDir" ImageDir
	),

	fn readFromFile =
	(
		readRenderersList()
		readDirectories()	
	),
	
	fn writeToFile =
	(
		deleteFile INIFile
		writeRenderersList()
		writeDirectories()
	)
	
) -- end of s_RIBit_Settings struct
